==> vcremote-1.0.22.tgz: package/lib/server.js <==
/**
  * ******************************************************
  *                                                       *
  *   Copyright (C) Microsoft. All rights reserved.       *
  *                                                       *
  *******************************************************
  */
"use strict";
const express = require("express");
const logger = require("morgan");
const errorhandler = require("errorhandler");
const http = require("http");
const https = require("https");
const fs = require("fs");
const path = require("path");
const nconf = require("nconf");
const os = require("os");
const buildManager = require("./build-manager");
const debugManager = require("./debug-manager");
const HostSpecifics = require("./host-specifics");
const Certs = require("./darwin/darwin-certs");
const utils = require("vcremote-utils");
var resources = utils.ResourcesManager;
var Server;
(function (Server) {
    var isSecureServer = true;
    var tempCertName = "tempCertInRequest.pem";
    var serverInstance;
    var serverConf;
    var app = express();
    var errorShutdown;
    var shutdown;
    function start(conf) {
        app.set("port", conf.get("port"));
        app.set("serverDir", conf.get("serverDir"));
        app.use(logger("dev"));
        app.use(errorhandler());
        // Initialize the build manager (after our app settings are all setup)
        buildManager.init(conf);
        app.get("/", function (req, res) {
            // If starting vcremote in non-secure mode, then requests are valid.
            if (!isSecureServer) {
                res.status(200).send(resources.getStringForLanguage(req, "IndexPageContent", app.get("port")));
                return;
            }
            isClientAuthorized(req, res).
                then(function (isAuthorized) {
                // If a cert in 'Authorized-Clients' matched the cert in the request, then it is a valid request.
                if (isAuthorized) {
                    res.status(200).send(resources.getStringForLanguage(req, "IndexPageContent", app.get("port")));
                }
                else {
                    invalidRequest(req, res);
                }
            });
        });
        // These routes are fully secured through client cert verification:
        if (utils.UtilHelper.argToBool(conf.get("secure"))) {
            app.all("/projects", onlyAuthorizedClientRequest);
            app.all("/projects/*", onlyAuthorizedClientRequest);
            app.all("/debug/*", onlyAuthorizedClientRequest);
            app.all("/filesystem/*", onlyAuthorizedClientRequest);
        }
        app.post("/projects", buildManager.addNewProject);
        app.post("/projects/:id", buildManager.updateProject);
        app.post("/projects/:id/configs", buildManager.addProjectConfig);
        app.post("/projects/:id/build", buildManager.buildProject);
        app.post("/projects/:id/build/:cfg", buildManager.buildProject);
        app.get("/projects", buildManager.getProjects);
        app.get("/projects/devices", buildManager.getDevices);
        app.get("/projects/:id", buildManager.getProjectConfigs);
        app.get("/projects/:id/build/configs", buildManager.getProjectConfigs);
        app.get("/projects/:id/config/:cfg", buildManager.getProjectConfig);
        app.get("/projects/:id/build/:cfg", buildManager.retrieveBuild);
        app.get("/projects/:id/build/:cfg/log", buildManager.getBuildLog);
        app.get("/projects/:id/build/:cfg/install", rerouteInstallBuild);
        app.get("/projects/:id/build/:cfg/status", buildManager.getBuildStatus);
        app.get("/filesystem/open", buildManager.execOpen);
        app.post("/filesystem/getfiles", buildManager.getFiles);
        app.post("/filesystem/putfiles", buildManager.putFiles);
        app.post("/filesystem/deletefiles", buildManager.deleteFiles);
        app.get("/debug/setupForDebugging", debugManager.setupForDebugging);
        app.get("/debug/appRemotePath", debugManager.getAppRemotePath);
        return startupServer(conf, app).
            then(registerShutdownHooks).
            catch(function (err) {
            console.error(resources.getStringForLanguage(conf.get("lang"), "ServerStartFailed"), err);
            if (err.stack) {
                console.error(err.stack);
            }
            throw err;
        });
    }
    Server.start = start;
    function stop(callback) {
        process.removeListener("uncaughtException", errorShutdown);
        process.removeListener("SIGTERM", shutdown);
        process.removeListener("SIGINT", shutdown);
        if (serverInstance) {
            var tempInstance = serverInstance;
            serverInstance = null;
            tempInstance.close(callback);
        }
    }
    Server.stop = stop;
    Promise.defer = function () {
        var result = {};
        result.promise = new Promise(function (resolve, reject) {
            result.resolve = resolve;
            result.reject = reject;
        });
        return result;
    };
    function rerouteInstallBuild(req, res) {
        buildManager.installBuild(req, res);
        // Kick off plain http server to bypass secure 'install' and 'debug' calls.
        // Must be done to start remote debugging session, because the iOS machine deems our self-signed client certs 'invalid' since they are not signed by any CA.
        if (utils.UtilHelper.argToBool(nconf.get("secure"))) {
            stop(() => { });
            startupPlainHttpServer(nconf, app);
            console.log(resources.getStringForLanguage(nconf.get("lang"), "RestartAfterDeployMessage"));
        }
    }
    Server.rerouteInstallBuild = rerouteInstallBuild;
    function resetServerCert(conf) {
        return HostSpecifics.hostSpecifics.resetServerCert(conf);
    }
    Server.resetServerCert = resetServerCert;
    function startupServer(conf, app) {
        var isSsl = utils.UtilHelper.argToBool(conf.get("secure"));
        return isSsl ? startupHttpsServer(conf, app) : startupPlainHttpServer(conf, app);
    }
    function startupPlainHttpServer(conf, app) {
        isSecureServer = false;
        Certs.authorizedClientCertsPath = path.join(conf.get("serverDir"), "certs", "Authorized-Clients");
        return new Promise(function (resolve, reject) { resolve(http.createServer(app)); }).
            then(function (svr) {
            var deferred = new Promise.defer;
            svr.on("error", function (err) {
                deferred.reject(friendlyServerListenError(err, conf));
            });
            svr.listen(app.get("port"), function () {
                serverInstance = svr;
                serverConf = conf;
                console.log(resources.getStringForLanguage(conf.get("lang"), "InsecureServerStarted"), app.get("port"));
                deferred.resolve(svr);
            });
            return deferred.promise;
        });
    }
    function startupHttpsServer(conf, app) {
        isSecureServer = true;
        var generatedNewCerts = false;
        return HostSpecifics.hostSpecifics.initializeServerCerts(conf).
            then(function (certStore) {
            if (certStore.newCerts) {
                generatedNewCerts = true;
                conf.set("suppressVisualStudioMessage", true);
                return certStore;
            }
            return new Promise(function (resolve, reject) { resolve(certStore); });
        }).
            then(function (certStore) {
            var sslSettings = {
                key: certStore.getKey(),
                cert: certStore.getCert(),
                requestCert: true,
                rejectUnauthorized: false
            };
            return https.createServer(sslSettings, app);
        }).
            then(function (svr) {
            var deferred = new Promise.defer;
            svr.on("error", function (err) {
                if (generatedNewCerts) {
                    HostSpecifics.hostSpecifics.removeAllCertsSync(conf);
                }
                deferred.reject(friendlyServerListenError(err, conf));
            });
            svr.listen(app.get("port"), function () {
                serverInstance = svr;
                serverConf = conf;
                console.log(resources.getStringForLanguage(conf.get("lang"), "SecureServerStarted"), app.get("port"));
                deferred.resolve(svr);
            });
            return deferred.promise;
        });
    }
    function friendlyServerListenError(err, conf) {
        if (err.code === "EADDRINUSE") {
            return resources.getStringForLanguage(conf.get("lang"), "ServerPortInUse", conf.get("port"));
        }
        else {
            return err.toString();
        }
    }
    function registerShutdownHooks() {
        // We should have serverConf defined at this point, but just in case fall back to english
        var lang = serverConf && serverConf.get("lang") || "en";
        // It is strongly recommended in a NodeJs server to kill the process off on uncaughtException.
        errorShutdown = function (err) {
            console.error(resources.getStringForLanguage(lang, "UncaughtErrorShutdown"));
            console.error(err);
            console.error(err.stack);
            console.info(resources.getStringForLanguage(lang, "ServerShutdown"));
            buildManager.shutdown();
            debugManager.shutdown();
            process.exit(1);
        };
        process.on("uncaughtException", errorShutdown);
        // Opportunity to clean up builds on exit
        shutdown = function () {
            console.info(resources.getStringForLanguage(lang, "ServerShutdown"));
            // BUG: Currently if buildManager.shutdown() is called while a build log is being written, rimraf will throw an exception on windows
            buildManager.shutdown();
            debugManager.shutdown();
            serverInstance.close();
            process.exit(0);
        };
        process.on("SIGTERM", shutdown);
        process.on("SIGINT", shutdown);
    }
    function isClientAuthorized(req, res) {
        var isAuthorized = new Promise(function (resolve, reject) { resolve(false); });
        var clientCertificateInRequest;
        try {
            clientCertificateInRequest = req.socket.getPeerCertificate();
        }
        catch (_a) {
            return isAuthorized;
        }
        var tempCertPath = createTempCertFile(req, res, clientCertificateInRequest);
        const authorizedClients = fs.readdirSync(Certs.authorizedClientCertsPath, { withFileTypes: true });
        if (tempCertPath != null) {
            // Check each certificate(s) in 'Authorized-Clients' directory for a match.
            for (const file of authorizedClients) {
                var isPEMFile = path.extname(file.name).toLowerCase() == ".pem";
                if (isPEMFile) {
                    isAuthorized = isAuthorized.then(function (previouslyAuthorized) {
                        if (previouslyAuthorized) {
                            // If previously authorized, then pass true up stream.
                            return true;
                        }
                        else {
                            // If not previously authorized, then try to find a match.
                            return Certs.getFingerprint(path.join(Certs.authorizedClientCertsPath, file.name)).
                                then(function (authorizedClientHash) {
                                return Certs.getFingerprint(tempCertPath).
                                    then(function (clientHash) {
                                    // Compares authorizedClientHash to the hash for the client certificate in the request.
                                    if (authorizedClientHash == clientHash) {
                                        return true;
                                    }
                                    else {
                                        return false;
                                    }
                                });
                            });
                        }
                    });
                }
            }
        }
        return isAuthorized;
    }
    // clientCertificateInRequest.fingerprint only supports SHA1, but we must use SHA256 for comparison.
    // So we create tempCertInRequest.pem to extract SHA256 from the client certificate sent in the request.
    function createTempCertFile(req, res, clientCertificateInRequest) {
        var pemString = ['-----BEGIN CERTIFICATE-----', clientCertificateInRequest.raw.toString('base64'), '-----END CERTIFICATE-----'].join(os.EOL);
        var tempCertPath = path.join(Certs.authorizedClientCertsPath, "../", tempCertName);
        try {
            fs.writeFileSync(tempCertPath, pemString);
        }
        catch (error) {
            invalidRequest(req, res);
        }
        return tempCertPath;
    }
    function invalidRequest(req, res) {
        console.log("");
        console.error(resources.getStringForLanguage(nconf.get("lang"), "MatchingCertNotFound"));
        console.error(resources.getStringForLanguage(nconf.get("lang"), "TransferClientCertMessage"));
        res.status(401).send(resources.getStringForLanguage(req, "UnauthorizedClientRequest"));
    }
    function onlyAuthorizedClientRequest(req, res, next) {
        // Force validate 'install' and 'debug' calls to launch debugging sessions.
        var url = req.url;
        if (url.includes("install") || url.includes("debug")) {
            next();
            return;
        }
        else {
            isClientAuthorized(req, res).
                then(function (isAuthorized) {
                if (isAuthorized) {
                    // If a cert in 'Authorized-Clients' matched the cert in the request, then it is a valid request.
                    next();
                }
                else {
                    invalidRequest(req, res);
                }
            });
        }
    }
    ;
})(Server || (Server = {}));
module.exports = Server;

==> vcremote-1.0.23.tgz: package/lib/server.js <==
/**
  * ******************************************************
  *                                                       *
  *   Copyright (C) Microsoft. All rights reserved.       *
  *                                                       *
  *******************************************************
  */
"use strict";
const express = require("express");
const logger = require("morgan");
const errorhandler = require("errorhandler");
const http = require("http");
const https = require("https");
const fs = require("fs");
const path = require("path");
const nconf = require("nconf");
const buildManager = require("./build-manager");
const debugManager = require("./debug-manager");
const HostSpecifics = require("./host-specifics");
const Certs = require("./darwin/darwin-certs");
const utils = require("vcremote-utils");
var resources = utils.ResourcesManager;
var Server;
(function (Server) {
    var isSecureServer = true;
    var tempCertName = "tempCertInRequest.pem";
    var serverInstance;
    var serverConf;
    var app = express();
    var errorShutdown;
    var shutdown;
    function start(conf) {
        app.set("port", conf.get("port"));
        app.set("serverDir", conf.get("serverDir"));
        app.use(logger("dev"));
        app.use(errorhandler());
        // Initialize the build manager (after our app settings are all setup)
        buildManager.init(conf);
        app.get("/", function (req, res) {
            // If starting vcremote in non-secure mode, then requests are valid.
            if (!isSecureServer) {
                res.status(200).send(resources.getStringForLanguage(req, "IndexPageContent", app.get("port")));
                return;
            }
            isClientAuthorized(req, res).
                then(function (isAuthorized) {
                // If a cert in 'Authorized-Clients' matched the cert in the request, then it is a valid request.
                if (isAuthorized) {
                    res.status(200).send(resources.getStringForLanguage(req, "IndexPageContent", app.get("port")));
                }
                else {
                    invalidRequest(req, res);
                }
            });
        });
        // These routes are fully secured through client cert verification:
        if (utils.UtilHelper.argToBool(conf.get("secure"))) {
            app.all("/projects", onlyAuthorizedClientRequest);
            app.all("/projects/*", onlyAuthorizedClientRequest);
            app.all("/debug/*", onlyAuthorizedClientRequest);
            app.all("/filesystem/*", onlyAuthorizedClientRequest);
        }
        app.post("/projects", buildManager.addNewProject);
        app.post("/projects/:id", buildManager.updateProject);
        app.post("/projects/:id/configs", buildManager.addProjectConfig);
        app.post("/projects/:id/build", buildManager.buildProject);
        app.post("/projects/:id/build/:cfg", buildManager.buildProject);
        app.get("/projects", buildManager.getProjects);
        app.get("/projects/devices", buildManager.getDevices);
        app.get("/projects/:id", buildManager.getProjectConfigs);
        app.get("/projects/:id/build/configs", buildManager.getProjectConfigs);
        app.get("/projects/:id/config/:cfg", buildManager.getProjectConfig);
        app.get("/projects/:id/build/:cfg", buildManager.retrieveBuild);
        app.get("/projects/:id/build/:cfg/log", buildManager.getBuildLog);
        app.get("/projects/:id/build/:cfg/install", rerouteInstallBuild);
        app.get("/projects/:id/build/:cfg/status", buildManager.getBuildStatus);
        app.get("/filesystem/open", buildManager.execOpen);
        app.post("/filesystem/getfiles", buildManager.getFiles);
        app.post("/filesystem/putfiles", buildManager.putFiles);
        app.post("/filesystem/deletefiles", buildManager.deleteFiles);
        app.get("/debug/setupForDebugging", debugManager.setupForDebugging);
        app.get("/debug/appRemotePath", debugManager.getAppRemotePath);
        return startupServer(conf, app).
            then(registerShutdownHooks).
            catch(function (err) {
            console.error(resources.getStringForLanguage(conf.get("lang"), "ServerStartFailed"), err);
            if (err.stack) {
                console.error(err.stack);
            }
            throw err;
        });
    }
    Server.start = start;
    function stop(callback) {
        process.removeListener("uncaughtException", errorShutdown);
        process.removeListener("SIGTERM", shutdown);
        process.removeListener("SIGINT", shutdown);
        if (serverInstance) {
            var tempInstance = serverInstance;
            serverInstance = null;
            tempInstance.close(callback);
        }
    }
    Server.stop = stop;
    Promise.defer = function () {
        var result = {};
        result.promise = new Promise(function (resolve, reject) {
            result.resolve = resolve;
            result.reject = reject;
        });
        return result;
    };
    function rerouteInstallBuild(req, res) {
        buildManager.installBuild(req, res);
        // Kick off plain http server to bypass secure 'install' and 'debug' calls.
        // Must be done to start remote debugging session, because the iOS machine deems our self-signed client certs 'invalid' since they are not signed by any CA.
        if (utils.UtilHelper.argToBool(nconf.get("secure"))) {
            stop(() => { });
            startupPlainHttpServer(nconf, app);
            console.log(resources.getStringForLanguage(nconf.get("lang"), "RestartAfterDeployMessage"));
        }
    }
    Server.rerouteInstallBuild = rerouteInstallBuild;
    function resetServerCert(conf) {
        return HostSpecifics.hostSpecifics.resetServerCert(conf);
    }
    Server.resetServerCert = resetServerCert;
    function startupServer(conf, app) {
        var isSsl = utils.UtilHelper.argToBool(conf.get("secure"));
        return isSsl ? startupHttpsServer(conf, app) : startupPlainHttpServer(conf, app);
    }
    function startupPlainHttpServer(conf, app) {
        isSecureServer = false;
        Certs.authorizedClientCertsPath = path.join(conf.get("serverDir"), "certs", "Authorized-Clients");
        return new Promise(function (resolve, reject) { resolve(http.createServer(app)); }).
            then(function (svr) {
            var deferred = new Promise.defer;
            svr.on("error", function (err) {
                deferred.reject(friendlyServerListenError(err, conf));
            });
            svr.listen(app.get("port"), function () {
                serverInstance = svr;
                serverConf = conf;
                console.log(resources.getStringForLanguage(conf.get("lang"), "InsecureServerStarted"), app.get("port"));
                deferred.resolve(svr);
            });
            return deferred.promise;
        });
    }
    function startupHttpsServer(conf, app) {
        isSecureServer = true;
        var generatedNewCerts = false;
        return HostSpecifics.hostSpecifics.initializeServerCerts(conf).
            then(function (certStore) {
            if (certStore.newCerts) {
                generatedNewCerts = true;
                conf.set("suppressVisualStudioMessage", true);
                return certStore;
            }
            return new Promise(function (resolve, reject) { resolve(certStore); });
        }).
            then(function (certStore) {
            var sslSettings = {
                key: certStore.getKey(),
                cert: certStore.getCert(),
                requestCert: true,
                rejectUnauthorized: false
            };
            return https.createServer(sslSettings, app);
        }).
            then(function (svr) {
            var deferred = new Promise.defer;
            svr.on("error", function (err) {
                if (generatedNewCerts) {
                    HostSpecifics.hostSpecifics.removeAllCertsSync(conf);
                }
                deferred.reject(friendlyServerListenError(err, conf));
            });
            svr.listen(app.get("port"), function () {
                serverInstance = svr;
                serverConf = conf;
                console.log(resources.getStringForLanguage(conf.get("lang"), "SecureServerStarted"), app.get("port"));
                deferred.resolve(svr);
            });
            return deferred.promise;
        });
    }
    function friendlyServerListenError(err, conf) {
        if (err.code === "EADDRINUSE") {
            return resources.getStringForLanguage(conf.get("lang"), "ServerPortInUse", conf.get("port"));
        }
        else {
            return err.toString();
        }
    }
    function registerShutdownHooks() {
        // We should have serverConf defined at this point, but just in case fall back to english
        var lang = serverConf && serverConf.get("lang") || "en";
        // It is strongly recommended in a NodeJs server to kill the process off on uncaughtException.
        errorShutdown = function (err) {
            console.error(resources.getStringForLanguage(lang, "UncaughtErrorShutdown"));
            console.error(err);
            console.error(err.stack);
            console.info(resources.getStringForLanguage(lang, "ServerShutdown"));
            buildManager.shutdown();
            debugManager.shutdown();
            process.exit(1);
        };
        process.on("uncaughtException", errorShutdown);
        // Opportunity to clean up builds on exit
        shutdown = function () {
            console.info(resources.getStringForLanguage(lang, "ServerShutdown"));
            // BUG: Currently if buildManager.shutdown() is called while a build log is being written, rimraf will throw an exception on windows
            buildManager.shutdown();
            debugManager.shutdown();
            serverInstance.close();
            process.exit(0);
        };
        process.on("SIGTERM", shutdown);
        process.on("SIGINT", shutdown);
    }
    function isClientAuthorized(req, res) {
        var isAuthorized = new Promise(function (resolve, reject) { resolve(false); });
        var clientCertificateInRequest;
        try {
            clientCertificateInRequest = req.socket.getPeerCertificate();
        }
        catch (_a) {
            return isAuthorized;
        }
        var tempCertPath = createTempCertFile(req, res, clientCertificateInRequest);
        const authorizedClients = fs.readdirSync(Certs.authorizedClientCertsPath, { withFileTypes: true });
        if (tempCertPath != null) {
            // Check each certificate(s) in 'Authorized-Clients' directory for a match.
            for (const file of authorizedClients) {
                var isPEMFile = path.extname(file.name).toLowerCase() == ".pem";
                if (isPEMFile) {
                    isAuthorized = isAuthorized.then(function (previouslyAuthorized) {
                        if (previouslyAuthorized) {
                            // If previously authorized, then pass true up stream.
                            return true;
                        }
                        else {
                            // If not previously authorized, then try to find a match.
                            return Certs.getFingerprint(path.join(Certs.authorizedClientCertsPath, file.name)).
                                then(function (authorizedClientHash) {
                                return Certs.getFingerprint(tempCertPath).
                                    then(function (clientHash) {
                                    // Compares authorizedClientHash to the hash for the client certificate in the request.
                                    if (authorizedClientHash == clientHash) {
                                        return true;
                                    }
                                    else {
                                        return false;
                                    }
                                });
                            });
                        }
                    });
                }
            }
        }
        return isAuthorized;
    }
    // clientCertificateInRequest.fingerprint only supports SHA1, but we must use SHA256 for comparison.
    // So we create tempCertInRequest.pem to extract SHA256 from the client certificate sent in the request.
    function createTempCertFile(req, res, clientCertificateInRequest) {
        var pemString = "-----BEGIN CERTIFICATE-----\n" + clientCertificateInRequest.raw.toString('base64') + "\n-----END CERTIFICATE-----\n";
        var tempCertPath = path.join(Certs.authorizedClientCertsPath, "../", tempCertName);
        try {
            fs.writeFileSync(tempCertPath, pemString);
        }
        catch (error) {
            invalidRequest(req, res);
        }
        return tempCertPath;
    }
    function invalidRequest(req, res) {
        console.log("");
        console.error(resources.getStringForLanguage(nconf.get("lang"), "MatchingCertNotFound"));
        console.error(resources.getStringForLanguage(nconf.get("lang"), "TransferClientCertMessage"));
        res.status(401).send(resources.getStringForLanguage(req, "UnauthorizedClientRequest"));
    }
    function onlyAuthorizedClientRequest(req, res, next) {
        // Force validate 'install' and 'debug' calls to launch debugging sessions.
        var url = req.url;
        if (url.includes("install") || url.includes("debug")) {
            next();
            return;
        }
        else {
            isClientAuthorized(req, res).
                then(function (isAuthorized) {
                if (isAuthorized) {
                    // If a cert in 'Authorized-Clients' matched the cert in the request, then it is a valid request.
                    next();
                }
                else {
                    invalidRequest(req, res);
                }
            });
        }
    }
    ;
})(Server || (Server = {}));
module.exports = Server;
